@@ -20,9 +20,26 @@ class AmiciCxxCodePrinter(CXX11CodePrinter):
             return code
         except TypeError as e:
             raise ValueError(
-                f'Encountered unsupported function in expression "{expr}": '
-                f'{e}!'
-            )
+                f'Encountered unsupported function in expression "{expr}"'
+            ) from e
+
+    def _print_min_max(self, expr, cpp_fun: str, sympy_fun):
+        # C++ doesn't like mixing int and double for arguments for min/max,
+        #  therefore, we just always convert to float
+        arg0 = sp.Float(expr.args[0]) if expr.args[0].is_number \
+            else expr.args[0]
+        if len(expr.args) == 1:
+            return self._print(arg0)
+        return "%s%s(%s, %s)" % (self._ns, cpp_fun, self._print(arg0),
+                                 self._print(sympy_fun(*expr.args[1:])))
+
+    def _print_Min(self, expr):
+        from sympy.functions.elementary.miscellaneous import Min
+        return self._print_min_max(expr, "min", Min)
+
+    def _print_Max(self, expr):
+        from sympy.functions.elementary.miscellaneous import Max
+        return self._print_min_max(expr, "max", Max)
 
     def _get_sym_lines_array(
             self,
