@@ -8,12 +8,18 @@
 
 namespace amici {
 
-AmiException::AmiException(const char *fmt, ...) {
+AmiException::AmiException()
+{
+    storeBacktrace(12);
+}
+
+AmiException::AmiException(const char *fmt, ...)
+    : AmiException()
+{
     va_list ap;
     va_start(ap, fmt);
-    vsnprintf(msg_.data(), msg_.size(), fmt, ap);
+    storeMessage(fmt, ap);
     va_end(ap);
-    storeBacktrace(12);
 }
 
 const char *AmiException::what() const noexcept {
@@ -29,6 +35,11 @@ void AmiException::storeBacktrace(const int nMaxFrames) {
              backtraceString(nMaxFrames).c_str());
 }
 
+void AmiException::storeMessage(const char *fmt, va_list argptr)
+{
+    vsnprintf(msg_.data(), msg_.size(), fmt, argptr);
+}
+
 CvodeException::CvodeException(const int error_code, const char *function) :
     AmiException("Cvode routine %s failed with error code %i",function,error_code){}
 
@@ -48,4 +59,12 @@ NewtonFailure::NewtonFailure(int code, const char *function) :
     error_code = code;
 }
 
+SetupFailure::SetupFailure(const char *fmt, ...)
+{
+    va_list ap;
+    va_start(ap, fmt);
+    storeMessage(fmt, ap);
+    va_end(ap);
+}
+
 } // namespace amici
