@@ -617,11 +617,15 @@ std::vector<bool> const &Model::getStateIsNonNegative() const {
 void Model::setStateIsNonNegative(std::vector<bool> const &nonNegative) {
     auto any_state_non_negative = std::any_of(nonNegative.begin(),
                                               nonNegative.end(),
-                                              [](bool x) { return x; });
-
-    if (any_state_non_negative && nx_solver != nx_rdata) {
-        throw AmiException("Non-negative states are not supported with"
-                           " conservation laws enabled");
+                                              [](bool x) {
+                                                  return x; });
+    if (nx_solver != nx_rdata) {
+        if(any_state_non_negative)
+            throw AmiException("Non-negative states are not supported with"
+                               " conservation laws enabled.");
+        // nothing to do, as `state_is_non_negative_` will always be all-false
+        // in case of conservation laws
+        return;
     }
     if (state_is_non_negative_.size() != static_cast<unsigned long>(nx_rdata)) {
         throw AmiException("Dimension of input stateIsNonNegative (%u) does "
